; Author:      Roy Klein Gebbinck (www.b-k-g.nl)
; Version:     20180209: First version.
; Created for: https://forum.bricsys.com/discussion/33426/lisp-that-draws-specific-block-with-parameters-to-coordinates-from-file

;;; ======================================================================
;;; Lib function: ProcessPoints_ProcessData
;;; Purpose:      Process a list of point records.
;;; Arguments:    dataStrLst  - List of strings.
;;;                             Each string represents a point record. Do not include headers.
;;;               fldSep      - Field separator as string. Typically "," or ";" or "\t" (tab).
;;;               decSep      - Decimal separator as string. Typically "." or ",".
;;;                             Only used for point coordinates.
;;;               idxPos      - Integer or nil.
;;;                             Integer:
;;;                               Field position (zero-based) of the point index.
;;;                             Nil:
;;;                               Do not include the point index.
;;;               coordPosLst - List or nil.
;;;                             List:
;;;                               3 integers or reals representing the X, Y and Z coordinates of the
;;;                               point.
;;;                               - An integer indicates a field position (zero-based) to be used
;;;                                 for that coordinate.
;;;                               - A real results in a fixed coordinate value.
;;;                             Nil:
;;;                               Do not include the coordinates of the point.
;;;               attPosLst   - List or nil.
;;;                             List of integers, strings and/or nils:
;;;                               - An integer indicates a field position (zero-based) to be used
;;;                                 for that attribute.
;;;                               - A string results in a fixed attribute value.
;;;                               - A nil means the attribute should be skipped.
;;;                             Nil:
;;;                               Do not include a list of attribute values.
;;; Return value: List with a sublist for each record.
;;; ======================================================================
(defun ProcessPoints_ProcessData (dataStrLst fldSep decSep idxPos coordPosLst attPosLst)
  (mapcar
    '(lambda (str / lst)
      (setq lst (vle-string-split fldSep str))
      (append
        (if idxPos
          (list (nth idxPos lst))
        )
        (if coordPosLst ; Items in list can be an integer or a real (= fixed coordinate).
          (list
            (mapcar
              '(lambda (pos) (if (vle-integerp pos) (read (vl-string-translate decSep "." (nth pos lst))) pos))
              coordPosLst
            )
          )
        )
        (if attPosLst ; Items in list can be an integer or a string (= fixed value) or nil (= skip attribute).
          (list
            (mapcar
              '(lambda (pos) (if (vle-integerp pos) (nth pos lst) pos))
              attPosLst
            )
          )
        )
      )
    )
    dataStrLst
  )
)

(defun ProcessPoints_Insert (spc blkNm pt scl ang attLst / blk)
  (setq blk
    (vla-insertblock
      spc
      pt    ; Insertion point.
      blkNm ; Name.
      scl   ; X scale.
      scl   ; Y scale.
      scl   ; Z Scale.
      ang   ; Rotation (in radians).
    )
  )
  (mapcar
    '(lambda (att str)
      (if str
        (vla-put-textstring att str)
      )
    )
    (vlax-invoke blk 'getattributes)
    attLst
  )
  blk
)

(defun c:Test ( / blkNm doc fnm lst spc)
  (setq doc (vla-get-activedocument (vlax-get-acad-object)))
  (vla-endundomark doc)
  (vla-startundomark doc)
  (if
    (and
      (setq fnm (getfiled "Select CSV File" "" "csv" 0))
      (setq lst (vle-file->list fnm nil))
      (setq lst
        (ProcessPoints_ProcessData
          lst        ; List of strings.
          ","        ; Field separator.
          "."        ; Decimal separator.
          nil        ; Do not include the point index.
          '(1 2 0.0) ; Coordinates of the point. Because the 3rd item is a real, a fixed value will be used for the Z-coordinate.
          '(3 nil 6) ; The block has 3 attributes. The 2nd will be skipped.
        )
      )
    )
    (progn
      (setq spc (vla-get-modelspace doc))
      (setq blkNm "TestBlock") ; The block must exist in the drawing.
      (foreach sub lst
        (ProcessPoints_Insert
          spc        ; Space.
          blkNm      ; Block name.
          (car sub)  ; Point.
          1.0        ; Scale.
          0.0        ; Rotation.
          (cadr sub) ; List of attribute values.
        )
      )
      (princ "\nDone ")
    )
  )
  (vla-endundomark doc)
  (princ)
)
